#################################
#    CMFSubversionBrowser
#    Copyright (C) 2002-2004  kapil thangavelu <k_vertigo@objectrealms.net>
#
#    This product is free software; you can redistribute it and/or
#    modify it under the terms of the GNU Lesser General Public
#    License as published by the Free Software Foundation; either
#    version 2.1 of the License, or (at your option) any later version.
#
#    This product is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public
#    License along with this product; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#################################

"""
Purpose: CMF variant of SVN Browser object model
Author: kapil thangavelu <pluto@objectrealms.net>
$Id: CMFSubversionBrowser.py,v 1.3 2003/02/28 05:10:01 k_vertigo Exp $
"""

import pprint

# zope infrastructure
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent
from ComputedAttribute import ComputedAttribute
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from ZPublisher.BaseRequest import RequestContainer

# cmf infrastructure
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.DynamicType import DynamicType 
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _getViewFor
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Products.CMFCore.interfaces.Dynamic import DynamicType as IDynamicType

import SubversionBrowser as svnb

from SubversionBrowser import SubversionBrowser, SubversionDirectory,SubversionFile, \
     SVNDirectoryFactory, SVNFileFactory, svn_path_join, log

from Traversal import SubversionZopeTraversal
from TypeInfo import svn_file_fti, svn_directory_fti

class PortalView:
    
    index_html = None
    
    def __call__(self):
        '''
        Invokes the default view.
        '''
        view = _getViewFor(self)
        if getattr(aq_base(view), 'isDocTemp', 0):
            return apply(view, (self, self.REQUEST))
        else:
            return view()

    def view(self):
        '''
        invoke default view even if index_html is overwritten.
        '''
        return self()
    
class CMFSubversionDirectory(SubversionDirectory,
                             PortalView,
                             DynamicType,
                             SimpleItem):

    meta_type = 'CMF Subversion Directory'


    __implements__ = IDynamicType


    def restrictedTraverse( self, info ):
        # sigh, breadcrumb support.
        if isinstance(info, str):
            chain = self.aq_chain
            for c in chain:
                if isinstance(c, RequestContainer):
                    break
                if c.getId() == info:
                    return c
        return SimpleItem.restrictedTraverse(self, info)
                

class CMFSubversionFile(SubversionFile,
                        PortalView,
                        DynamicType,
                        SimpleItem):

    meta_type = 'CMF Subversion File'

    __implements__ = IDynamicType

    def getAnnotated(self, revision_set=None):

        if revision_set is None: # sigh cheesy
            revision_set = ()
        else:
            try: revision_set = map(int, revision_set)
            except: revision_set = ()
                
        lines = self.getAnnotatedLines(revision_set)
        if not lines:
            return """ Annotation Not Available """

        buf = []
        for l in lines:
            buf.append('%s %s :: %s'%(str(l[0]).ljust(3), l[2], l[1]))

        return ''.join(buf)
    
class CMFSubversionBrowser(SubversionBrowser,
                           SubversionZopeTraversal,
                           SkinnedFolder,
                           PortalContent,
                           DefaultDublinCoreImpl):

    meta_type = 'CMF Subversion Browser'

    security = ClassSecurityInfo()
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'editBrowser')

InitializeClass(CMFSubversionBrowser)    

#################################
# factories for cmf

class ConstructionNotAllowedError(Exception): pass

def null_factory(self, *args, **kw):
    raise ConstructionNotAllowedError("this type cannot be constructed")

def addCMFSubversionBrowser(self, id):
    """ adds svn browsers """
    id = str(id)
    ob = CMFSubversionBrowser(id)
    self._setObject(id, ob)
    
#################################
# factories for browser

def CMFSVNDirectoryFactory(svnpath, klass=CMFSubversionDirectory):
    ob = SVNDirectoryFactory(svnpath, klass)
    ob._setPortalTypeName(svn_directory_fti['id'])
    return ob

def CMFSVNFileFactory(svnpath, klass=CMFSubversionFile):
    ob = SVNFileFactory(svnpath, klass)
    ob._setPortalTypeName(svn_file_fti['id'])
    return ob

svnb.SVNDirectoryFactory = CMFSVNDirectoryFactory
svnb.SVNFileFactory = CMFSVNFileFactory

#################################
# utilities

def invoke_view(self, name=None, view=None):
    if not name is None:
        view = getattr(self, name)
    else:
        view = getattr(self, 'view')        
    if getattr(aq_base(view), 'isDocTemp', 0):
        return apply(view, (self, self.REQUEST))
    else:
        return view()

